#
# License: BSD
#    https://raw.githubusercontent.com/stonier/groot_ansible/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Common methods.
"""

##############################################################################
# Imports
##############################################################################

import argparse
import functools
import os
import subprocess

from . import console

##############################################################################
# Methods
##############################################################################


def get_playbook_home():
    # This package is already imported and has a file attribute.  Use that.
    return os.path.abspath(os.path.join(os.path.dirname(__file__), 'playbooks'))

##############################################################################
# Ansible Command Formatting
##############################################################################


def append_verbosity_argument(cmd, verbosity):
    """
    Just in case we want to modify how many v's we use, centralise this here.
    """
    return cmd + " -vvvv" if verbosity else cmd

##############################################################################
# Generic Playbook to Subcommand
##############################################################################


def execute_generic_subcommand(name, become_sudo, args):
    """
    :param str name: name of the subcommand and playbook yaml
    :param bool become_sudo: whether to run as sudo or not

    """
    console.banner("'{0}'".format(name))
    connection = "-c local"
    list_tasks = "--list-tasks" if args.list_tasks else ""
    sudo_command = "-K" if become_sudo else ""
    cmd = "ansible-playbook {name}.yaml {sudo_command} -i localhost, {connection} {list_tasks}".format(**locals())
    cmd = append_verbosity_argument(cmd, args.verbose)
    console.key_value_pairs("Ansible", {"Command": cmd}, 10)
    print("")
    subprocess.call(cmd, cwd=args.home, shell=True)


def add_generic_subparser(subparsers, name, playbook_name, short_description, description, become_sudo):
    """
    Create a generic subparser which runs a playbook directly.

    :param subparsers: the subparsers factory from the parent argparser
    :param str name: name of the subcommand and playbook yaml
    :param str short_description: subcommand help string (shows in the parent parser)
    :param str description: full subcommand description (shows in --help for this subcommand)
    :param bool become_sudo: whether to run as sudo or not
    """
    parser = subparsers.add_parser(name,
                                   help=short_description,
                                   description=description,
                                   formatter_class=argparse.ArgumentDefaultsHelpFormatter
                                   )
    add_ansible_arguments(parser)
    parser.set_defaults(func=functools.partial(execute_generic_subcommand, playbook_name, become_sudo))


##############################################################################
# Argument Parsing
##############################################################################


def add_ansible_arguments(parser):
    """
    Any arguments for the parser that should be available for every single
    ansible call.

    :param argparse.ArgumentParser parser:
    """
    group = parser.add_argument_group(title="ansible arguments")
    group.add_argument('-v', '--verbose', action='store_true', help='execute the playbook with extra verbosity')
    group.add_argument('--home', action='store', default=get_playbook_home(), help='path to the groot-ansible playbooks home')
    group.add_argument('--list-tasks', action='store_true', default=False, help='ansible will list the task that would be executed by this playbook')


def add_apt_arguments(parser):
    """
    Any arguments for the parser related to apt management.

    :param argparse.ArgumentParser parser:
    """
    group = parser.add_argument_group(title="apt arguments")
    group.add_argument('-s', '--skip-apt-update', action='store_true', help='skip an update of the apt cache')



#!/usr/bin/env python3

##############################################################################
# Library
##############################################################################

# from ansible.module_utils.basic import * #@IgnorePep8

import ansible.module_utils.basic
import os
import subprocess

##############################################################################
# Documentation
##############################################################################

DOCUMENTATION = '''
---
module: vci
short_description: Find and import vcs repos from an index
'''

EXAMPLES = '''
- name: Find and import a vcs workspace
  vci:
    key: catkin
    path: /tmp/foo

- name: Find, import or update (if exists) a vcs workspace
  vci:
    key: catkin
    path: /tmp/foo
    force: true

- name: Find, import from a specified index
  vci:
    key: catkin
    index: https://raw.githubusercontent.com/stonier/vci/repos/kinetic.yaml
    path: /tmp/foo
'''

##############################################################################
# Implementation
##############################################################################


def main():
    module = ansible.module_utils.basic.AnsibleModule(
        argument_spec  = dict(                                                        #@IgnorePep8
            key        = dict(required=True, type='str'),                             #@IgnorePep8
            index      = dict(default='',    type='str'),                             #@IgnorePep8
            path       = dict(default='.',   type='str'),                             #@IgnorePep8
            force      = dict(default=False, type='bool'),                            #@IgnorePep8
            # pull      = dict(default=True, type='bool'),                            #@IgnorePep8
        )
    )
    abs_path = os.path.abspath(module.params["path"])
    if os.path.exists(abs_path):
        if not module.params["force"]:
            module.exit_json(changed=False,
                             msg="already exists and 'force' is not requested",
                             index=module.params["index"],
                             yaml="",
                             output="",
                             parameters=module.params)
    else:
        os.makedirs(abs_path)

    if not module.params["index"]:
        try:
            cmd = "vci config --no-colour"
            index = subprocess.check_output(cmd, shell=True)
        except subprocess.CalledProcessError as e:
            module.fail_json(msg="Failed to retrieve current index [{0}]".format(str(e)),
                             cmd=cmd,
                             index=module.params['index'],
                             yaml="",
                             output=e.output,
                             parameters=module.params
                             )
            return
    else:
        index = module.params['index']

    try:
        cmd = "vci find --index={index} {key}".format(index=index, key=module.params["key"])
        yaml = subprocess.check_output(cmd, cwd=abs_path, shell=True)
    except subprocess.CalledProcessError as e:
        module.fail_json(msg="Failed to find key '{0}'".format(module.params['key']),
                         cmd=cmd,
                         index=index,
                         yaml="",
                         output=e.output,
                         parameters=module.params)
        return

    cmd = "vci find --index={index} {key} | vcs import".format(index=index, key=module.params["key"])
    changed = False
    try:
        out = subprocess.check_output(cmd, cwd=abs_path, shell=True)
        # subprocess now gives you bytes, not strings, in python3
        # make sure you operate with bytes objects (or decode the original result)
        if any(keyword in out for keyword in [b"Cloning", b"behind"]):
            changed = True
    except subprocess.CalledProcessError as e:
        module.fail_json(msg="Failed to import key '{0}'".format(module.params['key']),
                         cmd=cmd,
                         index=index,
                         yaml=yaml,
                         output=e.output,
                         parameters=module.params)
        return
    else:
        module.exit_json(changed=changed,
                         index=index,
                         yaml=yaml,
                         output=out,
                         parameters=module.params)
    # TODO : implement a pull


if __name__ == '__main__':
    main()



#
# License: BSD
#    https://raw.githubusercontent.com/stonier/groot_ansible/devel/LICENSE
#

##############################################################################
# Description
##############################################################################

"""
.. module:: console
   :synopsis: Tools for colourising the console.

Colour definitions and logging functions for colourising the console.

----

**Colour Definitions**

The current list of colour definitions include:

 * ``Regular``: black, red, green, yellow, blue, magenta, cyan, white,
 * ``Bold``: bold, bold_black, bold_red, bold_green, bold_yellow, bold_blue, bold_magenta, bold_cyan, bold_white

These colour definitions can be used in the following way:

.. code-block:: python

   import rocon_console.console as console
   print(console.cyan + "    Name" + console.reset + ": " + console.yellow + "Dude" + console.reset)

"""

##############################################################################
# Imports
##############################################################################

import sys

##############################################################################
# Methods
##############################################################################


def console_has_colours(stream):
    """
    Detects if the specified stream has colourising capability.

    :param stream: stream to check (typically sys.stdout)
    """
    if not hasattr(stream, "isatty"):
        return False
    if not stream.isatty():
        return False  # auto color only on TTYs
    try:
        import curses
        curses.setupterm()
        return curses.tigetnum("colors") > 2
    except:
        # guess false in case of error
        return False

has_colours = console_has_colours(sys.stdout)
if has_colours:
    # reset = "\x1b[0;0m"
    reset = "\x1b[0m"
    bold = "\x1b[%sm" % '1'
    black, red, green, yellow, blue, magenta, cyan, white = ["\x1b[%sm" % str(i) for i in range(30, 38)]
    bold_black, bold_red, bold_green, bold_yellow, bold_blue, bold_magenta, bold_cyan, bold_white = ["\x1b[%sm" % ('1;' + str(i)) for i in range(30, 38)]
else:
    reset = ""
    bold = ""
    black, red, green, yellow, blue, magenta, cyan, white = ["" for i in range(30, 38)]
    bold_black, bold_red, bold_green, bold_yellow, bold_blue, bold_magenta, bold_cyan, bold_white = ["" for i in range(30, 38)]

colours = [bold,
           black, red, green, yellow, blue, magenta, cyan, white,
           bold_black, bold_red, bold_green, bold_yellow, bold_blue, bold_magenta, bold_cyan, bold_white
           ]


def pretty_print(msg, colour=white):
    if has_colours:
        seq = colour + msg + reset
        sys.stdout.write(seq)
    else:
        sys.stdout.write(msg)


def pretty_println(msg, colour=white):
    if has_colours:
        seq = colour + msg + reset
        sys.stdout.write(seq)
        sys.stdout.write("\n")
    else:
        sys.stdout.write(msg)


##############################################################################
# Console
##############################################################################


def debug(msg):
    print(green + msg + reset)


def warning(msg):
    print(yellow + msg + reset)


def info(msg):
    print(msg)


def error(msg):
    print(red + msg + reset)


def logdebug(message):
    '''
    Prefixes '[DEBUG]' and colours the message green.

    :param message str: message to log.
    '''
    print(green + "[DEBUG] " + message + reset)


def loginfo(message):
    '''
    Prefixes '[INFO]' to the message.

    :param message str: message to log.
    '''
    print("[INFO ] " + message)


def logwarn(message):
    '''
    Prefixes '[WARN ]' and colours the message yellow.

    :param message str: message to log.
    '''
    print(yellow + "[WARN ] " + message + reset)


def logerror(message):
    '''
    Prefixes '[ERROR]' and colours the message red.

    :param message str: message to log.
    '''
    print(red + "[ERROR] " + message + reset)


def logfatal(message):
    '''
    Prefixes '[fatal]' and colours the message bold red.

    :param message str: message to log.
    '''
    print(bold_red + "[error] " + message + reset)


def _colorise_key_value_pairs(key_value_strings, width=None):
    colorised = {}
    if width is None:
        width = max(len(k) for k in key_value_strings.keys())
    for key, value in key_value_strings.items():
        colorised_key = cyan + "{key:{width}}".format(key=key, width=width) + reset
        colorised_value = yellow + value + reset
        colorised[colorised_key] = colorised_value
    return colorised


def key_value_pairs(title, key_value_strings, width=None):
    print(green + title + reset)
    for k, v in _colorise_key_value_pairs(key_value_strings, width).items():
        print("  " + k + ": " + v)


def banner(title):
    width = 80
    print("")
    print(green + "*" * width + reset)
    print(bold + "{text:^{width}}".format(text=title, width=width) + reset)
    print(green + "*" * width + reset)
    print("")

##############################################################################
# Main
##############################################################################

if __name__ == '__main__':
    for colour in colours:
        pretty_print("dude\n", colour)
    logdebug("loginfo message")
    logwarn("logwarn message")
    logerror("logerror message")
    logfatal("logfatal message")
    pretty_print("red\n", red)
    print("some normal text")

    print(cyan + "    Name" + reset + ": " + yellow + "Dude" + reset)



#
# License: BSD
#    https://raw.githubusercontent.com/stonier/groot_ansible/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Python wrapped ansible scripts for installing and updating systems for
various use cases.
"""

##############################################################################
# Imports
##############################################################################

import argparse

from . import common
from . import console
from . import ros
from . import update
from . import testies
from . import workstation

##############################################################################
# Constants
##############################################################################

__version__ = '0.4.3'

##############################################################################
# Main
##############################################################################


def version_string():
    return console.cyan + "Version" + console.reset + " : " + console.yellow + __version__ + console.reset


def main(args=None):
    """
    Entry point to the console 'groot-ansible' tool.
    """
    try:
        parser = argparse.ArgumentParser(
            description=console.green + "A python frontend to groot playbooks." + console.reset,
            epilog=console.bold + console.white + "And his noodly appendage reached forth to tickle the blessed...\n" + console.reset,
            formatter_class=argparse.RawDescriptionHelpFormatter)
        parser.add_argument('-v', '--version', action='version', version=version_string())

        subparsers = parser.add_subparsers(title='commands',
                                           help='valid commands for groot-ansible interactions')
        update.add_subparser(subparsers)
        testies.add_subparser(subparsers)

        common.add_generic_subparser(subparsers,
                                     name="bootstrap/pc",
                                     playbook_name="bootstrap-pc",
                                     short_description="bootstrap a pc/laptop for development",
                                     description="Standard ubuntu development workstation",
                                     become_sudo=True)
        workstation.add_subparser(subparsers)

        common.add_generic_subparser(subparsers, name="os/ubuntu",
                                     playbook_name="os-ubuntu",
                                     short_description="useful non-core packages for ubuntu",
                                     description="Extra packages and configuration for the core of a basic development environment",
                                     become_sudo=True)
        common.add_generic_subparser(subparsers, name="os/kubuntu",
                                     playbook_name="os-kubuntu",
                                     short_description="kubuntu desktop packages and configuration",
                                     description="Add the kubuntu (full) desktop environment to an ubuntu installation",
                                     become_sudo=True)
        common.add_generic_subparser(subparsers, name="os/system76",
                                     playbook_name="os-system76",
                                     short_description="drivers from system76 for ubuntu",
                                     description="System76 environment setup/install/update",
                                     become_sudo=True)

        common.add_generic_subparser(subparsers, name="devel/git",
                                     playbook_name="devel-git",
                                     short_description="git binaries, modules and configuration",
                                     description="Git binaries, modules (lfs) and user configuration",
                                     become_sudo=True)
        common.add_generic_subparser(subparsers, name="devel/powerline",
                                     playbook_name="devel-powerline",
                                     short_description="powerline in the shell for the user",
                                     description="Setup powerline in the shell for the user",
                                     become_sudo=True)
        ros.add_subparser(subparsers)
        common.add_generic_subparser(subparsers, name="devel/ros2",
                                     playbook_name="devel-ros2",
                                     short_description="ros2 environment for ubuntu",
                                     description="ROS 2 environment setup/install/update",
                                     become_sudo=True)
        
        common.add_generic_subparser(subparsers, name="extras/chrome",
                                     playbook_name="extras-chrome",
                                     short_description="google chrome for ubuntu",
                                     description="Google chrome setup/install/update",
                                     become_sudo=True)
        common.add_generic_subparser(subparsers, name="extras/vscode",
                                     playbook_name="extras-vscode",
                                     short_description="vscode for ubuntu",
                                     description="VSCode setup/install/update",
                                     become_sudo=True)
        common.add_generic_subparser(subparsers, name="extras/snorriheim",
                                     playbook_name="extras-snorriheim",
                                     short_description="snorriheim ppa and packages",
                                     description="Snorriheim's PPA and packages for ubuntu",
                                     become_sudo=True)
        options = parser.parse_args(args)
        # options, unused_unknown_args = parser.parse_known_args(args)
        options.func(options)  # relay arg parsing to the subparser configured `set_defaults` function callback

    except KeyboardInterrupt:
        print('Interrupted by user!')



#
# License: BSD
#    https://raw.githubusercontent.com/stonier/groot_ansible/devel/LICENSE
#
##############################################################################
# Documentation
##############################################################################

"""
Ansible subcommands for ros management.
"""

##############################################################################
# Imports
##############################################################################

import argparse
import subprocess

from . import common
from . import console

##############################################################################
# Methods
##############################################################################


def parse_args(args):
    """
    Updates ansible
    """
    console.banner("'update'")
    connection = "-c local"
    list_tasks = "--list-tasks" if args.list_tasks else ""
    cmd = "ansible-playbook utils-update.yaml -K -i localhost, {connection} {list_tasks}".format(**locals())
    cmd = common.append_verbosity_argument(cmd, args.verbose)
    console.key_value_pairs("Ansible", {"Command": cmd}, 10)
    print("")
    subprocess.call(cmd, cwd=args.home, shell=True)


def add_subparser(subparsers):
    """
    Add our own argparser to the parent.

    :param subparsers: the subparsers factory from the parent argparser.
    """
    parser = subparsers.add_parser("utils/update",
                                   description="Update apt and the groot_ansible ecosystem",  # this shows in the help for this command
                                   help="update your ansible/apt environment",  # this shows in the parent parser
                                   formatter_class=argparse.ArgumentDefaultsHelpFormatter
                                   )
    common.add_ansible_arguments(parser)
    parser.set_defaults(func=parse_args)



#!/usr/bin/python

##############################################################################
# Library
##############################################################################

# from ansible.module_utils.basic import * #@IgnorePep8

import ansible.module_utils.basic
import os
import subprocess

##############################################################################
# Documentation
##############################################################################

DOCUMENTATION = '''
---
module: vci_config
short_description: Configure a vci index url
'''

EXAMPLES = '''
- name: Configure uri with a url (do not override unless force is true)
  vci_config:
    index: https://raw.githubusercontent.com/stonier/vci/repos/kinetic.yaml
    force: true
'''

##############################################################################
# Implementation
##############################################################################


def main():
    module = ansible.module_utils.basic.AnsibleModule(
        argument_spec  = dict(                                                        #@IgnorePep8
            index      = dict(required=True, type='str'),                             #@IgnorePep8
            force      = dict(default=False, type='bool'),                            #@IgnorePep8
        )
    )
    try:
        cmd = "vci config --no-colour"
        index = subprocess.check_output(cmd, shell=True).rstrip()
        vci_configured = True
        print(f"VCI Configured: {index}")
    except subprocess.CalledProcessError as e:
        index = None
        vci_configured = False
        print(f"VCI Not Configured")

    print(f"Configure with {module.params['index']}")
    if not module.params["force"] and vci_configured:
        module.exit_json(msg="VCI already configured, refusing to override as requested",
                         changed=False,
                         index=index,
                         parameters=module.params)

    if index and (module.params['index'] == index):
        module.exit_json(msg="VCI already configured with the specified index",
                         changed=False,
                         index=index,
                         parameters=module.params)

    index = module.params['index']

    try:
        cmd = "vci config --no-colour {index}".format(index=index)
        yaml = subprocess.check_output(cmd, shell=True)
    except subprocess.CalledProcessError as e:
        module.fail_json(msg="Failed to configure the index '{}'".format(index),
                         cmd=cmd,
                         index=index,
                         yaml="",
                         output=e.output,
                         parameters=module.params)
        return

    module.exit_json(changed=True,
                        index=index,
                        yaml=yaml,
                        parameters=module.params)

if __name__ == '__main__':
    main()
